@@ -117,8 +117,15 @@ def run_zxpy(filename: str, module: ast.Module) -> None:
     """Runs zxpy on a given file"""
     patch_shell_commands(module)
     code = compile(module, filename, mode='exec')
-    # Passing `__name__` in locals() to avoid overriding any globals
-    exec(code, {'__name__': '__main__'})
+    exec(
+        code,
+        {
+            '__name__': '__main__',
+            'run_shell': run_shell,
+            'run_shell_alternate': run_shell_alternate,
+            'shlex_quote': shlex_quote,
+        },
+    )
 
 
 def patch_shell_commands(module: Union[ast.Module, ast.Interactive]) -> None:
@@ -126,15 +133,21 @@ def patch_shell_commands(module: Union[ast.Module, ast.Interactive]) -> None:
     shell_runner = ShellRunner()
     shell_runner.visit(module)
 
-    for statement in module.body:
-        print_shell_outputs(statement)
-
     ast.fix_missing_locations(module)
 
 
 def quote_fstring_args(fstring: ast.JoinedStr) -> None:
     for index, node in enumerate(fstring.values):
         if isinstance(node, ast.FormattedValue):
+            # If it's marked as a raw shell string, then don't escape
+            if (
+                isinstance(node.format_spec, ast.JoinedStr)
+                and len(node.format_spec.values) == 1
+                and node.format_spec.values[0].value == 'raw'
+            ):
+                node.format_spec = None
+                continue
+
             fstring.values[index] = ast.Call(
                 func=ast.Name(id='shlex_quote', ctx=ast.Load()),
                 args=[node],
@@ -148,6 +161,7 @@ class ShellRunner(ast.NodeTransformer):
     def modify_expr(
             expr: ast.expr,
             return_stderr_and_returncode: bool = False,
+            print_it: bool = False,
     ) -> ast.expr:
         if (
             isinstance(expr, ast.UnaryOp)
@@ -163,16 +177,26 @@ class ShellRunner(ast.NodeTransformer):
                 else 'run_shell'
             )
 
+            if print_it:
+                keywords = [
+                    ast.keyword(
+                        arg='print_it',
+                        value=ast.Constant(value=True),
+                    )
+                ]
+            else:
+                keywords = []
+
             return ast.Call(
                 func=ast.Name(id=function_name, ctx=ast.Load()),
                 args=[expr.operand],
-                keywords=[],
+                keywords=keywords,
             )
 
         return expr
 
     def visit_Expr(self, expr: ast.Expr) -> ast.Expr:
-        expr.value = self.modify_expr(expr.value)
+        expr.value = self.modify_expr(expr.value, print_it=True)
         super().generic_visit(expr)
         return expr
 
@@ -198,29 +222,6 @@ class ShellRunner(ast.NodeTransformer):
         return attr
 
 
-def print_shell_outputs(statement: ast.stmt) -> None:
-    """Set print_it to True on every top level run_shell"""
-    if isinstance(statement, ast.FunctionDef):
-        for substatement in statement.body:
-            print_shell_outputs(substatement)
-
-    if not isinstance(statement, ast.Expr):
-        return
-
-    expr = statement.value
-    if (
-        isinstance(expr, ast.Call)
-        and isinstance(expr.func, ast.Name)
-        and expr.func.id == 'run_shell'
-    ):
-        expr.keywords = [
-            ast.keyword(
-                arg='print_it',
-                value=ast.Constant(value=True),
-            )
-        ]
-
-
 def setup_zxpy_repl() -> None:
     """Sets up a zxpy interactive session"""
     print("zxpy shell")
